diff --git a/core/core.services.yml b/core/core.services.yml
index 90c40a9..0049ae8 100644
--- a/core/core.services.yml
+++ b/core/core.services.yml
@@ -118,15 +118,19 @@ services:
     arguments: ['@service_container', '@settings']
   keyvalue.database:
     class: Drupal\Core\KeyValueStore\KeyValueDatabaseFactory
-    arguments: ['@database']
+    arguments: ['@serialization.phpserialize', '@database']
   keyvalue.expirable:
     class: Drupal\Core\KeyValueStore\KeyValueExpirableFactory
     arguments: ['@service_container', '@settings']
   keyvalue.expirable.database:
     class: Drupal\Core\KeyValueStore\KeyValueDatabaseExpirableFactory
+    arguments: ['@serialization.phpserialize', '@database']
     tags:
       - { name: needs_destruction }
-    arguments: ['@database']
+
+  serialization.phpserialize:
+    class: Drupal\Core\Serialization\PhpSerialize
+
   settings:
     class: Drupal\Component\Utility\Settings
     factory_class: Drupal\Component\Utility\Settings
@@ -275,7 +279,7 @@ services:
     arguments: ['@database']
   user.tempstore:
     class: Drupal\user\TempStoreFactory
-    arguments: ['@database', '@lock']
+    arguments: ['@serialization.phpserialize', '@database', '@lock']
   router.request_context:
     class: Symfony\Component\Routing\RequestContext
     calls:
diff --git a/core/lib/Drupal/Core/KeyValueStore/DatabaseStorage.php b/core/lib/Drupal/Core/KeyValueStore/DatabaseStorage.php
index 1464186..d25d2f3 100644
--- a/core/lib/Drupal/Core/KeyValueStore/DatabaseStorage.php
+++ b/core/lib/Drupal/Core/KeyValueStore/DatabaseStorage.php
@@ -9,6 +9,7 @@
 
 use Drupal\Core\Database\Query\Merge;
 use Drupal\Core\Database\Connection;
+use Drupal\Core\Serialization\SerializationInterface;
 
 /**
  * Defines a default key/value store implementation.
@@ -19,6 +20,13 @@
 class DatabaseStorage extends StorageBase {
 
   /**
+   * The serialization class to use.
+   *
+   * @var \Drupal\Core\Serialization\SerializationInterface
+   */
+  protected $serializer;
+
+  /**
    * The database connection.
    *
    * @var \Drupal\Core\Database\Connection
@@ -37,11 +45,16 @@ class DatabaseStorage extends StorageBase {
    *
    * @param string $collection
    *   The name of the collection holding key and value pairs.
+   * @param \Drupal\Core\Serialization\SerializationInterface $serializer
+   *   The serialization class to use.
+   * @param \Drupal\Core\Database\Connection $connection
+   *   The database connection to use.
    * @param string $table
    *   The name of the SQL table to use, defaults to key_value.
    */
-  public function __construct($collection, Connection $connection, $table = 'key_value') {
+  public function __construct($collection, SerializationInterface $serializer, Connection $connection, $table = 'key_value') {
     parent::__construct($collection);
+    $this->serializer = $serializer;
     $this->connection = $connection;
     $this->table = $table;
   }
@@ -65,7 +78,7 @@ public function getMultiple(array $keys) {
       $result = $this->connection->query('SELECT name, value FROM {' . $this->connection->escapeTable($this->table) . '} WHERE name IN (:keys) AND collection = :collection', array(':keys' => $keys, ':collection' => $this->collection))->fetchAllAssoc('name');
       foreach ($keys as $key) {
         if (isset($result[$key])) {
-          $values[$key] = unserialize($result[$key]->value);
+          $values[$key] = $this->serializer->decode($result[$key]->value);
         }
       }
     }
@@ -86,7 +99,7 @@ public function getAll() {
 
     foreach ($result as $item) {
       if ($item) {
-        $values[$item->name] = unserialize($item->value);
+        $values[$item->name] = $this->serializer->decode($item->value);
       }
     }
     return $values;
@@ -101,7 +114,7 @@ public function set($key, $value) {
         'name' => $key,
         'collection' => $this->collection,
       ))
-      ->fields(array('value' => serialize($value)))
+      ->fields(array('value' => $this->serializer->encode($value)))
       ->execute();
   }
 
@@ -113,7 +126,7 @@ public function setIfNotExists($key, $value) {
       ->insertFields(array(
         'collection' => $this->collection,
         'name' => $key,
-        'value' => serialize($value),
+        'value' => $this->serializer->encode($value),
       ))
       ->condition('collection', $this->collection)
       ->condition('name', $key)
diff --git a/core/lib/Drupal/Core/KeyValueStore/KeyValueDatabaseExpirableFactory.php b/core/lib/Drupal/Core/KeyValueStore/KeyValueDatabaseExpirableFactory.php
index 9000e20..895d7ed 100644
--- a/core/lib/Drupal/Core/KeyValueStore/KeyValueDatabaseExpirableFactory.php
+++ b/core/lib/Drupal/Core/KeyValueStore/KeyValueDatabaseExpirableFactory.php
@@ -9,6 +9,7 @@
 
 use Drupal\Core\DestructableInterface;
 use Drupal\Core\Database\Connection;
+use Drupal\Core\Serialization\SerializationInterface;
 
 /**
  * Defines the key/value store factory for the database backend.
@@ -23,6 +24,13 @@ class KeyValueDatabaseExpirableFactory implements KeyValueExpirableFactoryInterf
   protected $storages = array();
 
   /**
+   * The serialization class to use.
+   *
+   * @var \Drupal\Core\Serialization\SerializationInterface
+   */
+  protected $serializer;
+
+  /**
    * The database connection.
    *
    * @var \Drupal\Core\Database\Connection
@@ -32,11 +40,13 @@ class KeyValueDatabaseExpirableFactory implements KeyValueExpirableFactoryInterf
   /**
    * Constructs this factory object.
    *
-   *
+   * @param \Drupal\Core\Serialization\SerializationInterface $serializer
+   *   The serialization class to use.
    * @param \Drupal\Core\Database\Connection $connection
    *   The Connection object containing the key-value tables.
    */
-  function __construct(Connection $connection) {
+  function __construct(SerializationInterface $serializer, Connection $connection) {
+    $this->serializer = $serializer;
     $this->connection = $connection;
   }
 
@@ -45,7 +55,7 @@ function __construct(Connection $connection) {
    */
   public function get($collection) {
     if (!isset($this->storages[$collection])) {
-      $this->storages[$collection] = new DatabaseStorageExpirable($collection, $this->connection);
+      $this->storages[$collection] = new DatabaseStorageExpirable($collection, $this->serializer, $this->connection);
     }
     return $this->storages[$collection];
   }
diff --git a/core/lib/Drupal/Core/KeyValueStore/KeyValueDatabaseFactory.php b/core/lib/Drupal/Core/KeyValueStore/KeyValueDatabaseFactory.php
index 1a840b3..43063ab 100644
--- a/core/lib/Drupal/Core/KeyValueStore/KeyValueDatabaseFactory.php
+++ b/core/lib/Drupal/Core/KeyValueStore/KeyValueDatabaseFactory.php
@@ -6,8 +6,10 @@
  */
 
 namespace Drupal\Core\KeyValueStore;
+
 use Drupal\Core\Database\Connection;
 use Drupal\Core\Database\Database;
+use Drupal\Core\Serialization\SerializationInterface;
 
 /**
  * Defines the key/value store factory for the database backend.
@@ -15,13 +17,29 @@
 class KeyValueDatabaseFactory implements KeyValueFactoryInterface {
 
   /**
-   * Constructs this factory object.
+   * The serialization class to use.
+   *
+   * @var \Drupal\Core\Serialization\SerializationInterface
+   */
+  protected $serializer;
+
+  /**
+   * The database connection to use.
    *
+   * @var \Drupal\Core\Database\Connection
+   */
+  protected $connection;
+
+  /**
+   * Constructs this factory object.
    *
+   * @param \Drupal\Core\Serialization\SerializationInterface $serializer
+   *   The serialization class to use.
    * @param \Drupal\Core\Database\Connection $connection
    *   The Connection object containing the key-value tables.
    */
-  function __construct(Connection $connection) {
+  function __construct(SerializationInterface $serializer, Connection $connection) {
+    $this->serializer = $serializer;
     $this->connection = $connection;
   }
 
@@ -29,6 +47,6 @@ function __construct(Connection $connection) {
    * {@inheritdoc}
    */
   public function get($collection) {
-    return new DatabaseStorage($collection, $this->connection);
+    return new DatabaseStorage($collection, $this->serializer, $this->connection);
   }
 }
diff --git a/core/lib/Drupal/Core/Serialization/Exception/InvalidDataTypeException.php b/core/lib/Drupal/Core/Serialization/Exception/InvalidDataTypeException.php
new file mode 100644
index 0000000..e6b8ce6
--- /dev/null
+++ b/core/lib/Drupal/Core/Serialization/Exception/InvalidDataTypeException.php
@@ -0,0 +1,14 @@
+<?php
+
+/**
+ * @file
+ * Contains \Drupal\Core\Serialization\Exception\InvalidDataTypeException.
+ */
+
+namespace Drupal\Core\Serialization\Exception;
+
+/**
+ * Exception thrown when a data type is invalid.
+ */
+class InvalidDataTypeException extends \InvalidArgumentException {
+}
diff --git a/core/lib/Drupal/Core/Serialization/PhpSerialize.php b/core/lib/Drupal/Core/Serialization/PhpSerialize.php
new file mode 100644
index 0000000..4b912ba
--- /dev/null
+++ b/core/lib/Drupal/Core/Serialization/PhpSerialize.php
@@ -0,0 +1,36 @@
+<?php
+
+/**
+ * @file
+ * Contains \Drupal\Core\Serialization\PhpSerialize.
+ */
+
+namespace Drupal\Core\Serialization;
+
+/**
+ * Default serialization for serialized PHP.
+ */
+class PhpSerialize implements SerializationInterface {
+
+  /**
+   * {@inheritdoc}
+   */
+  public static function encode($data) {
+    return serialize($data);
+  }
+
+  /**
+   * {@inheritdoc}
+   */
+  public static function decode($raw) {
+    return unserialize($raw);
+  }
+
+  /**
+   * {@inheritdoc}
+   */
+  public static function getFileExtension() {
+    return 'serialized';
+  }
+
+}
diff --git a/core/lib/Drupal/Core/Serialization/SerializationInterface.php b/core/lib/Drupal/Core/Serialization/SerializationInterface.php
new file mode 100644
index 0000000..7713032
--- /dev/null
+++ b/core/lib/Drupal/Core/Serialization/SerializationInterface.php
@@ -0,0 +1,45 @@
+<?php
+
+/**
+ * @file
+ * Contains \Drupal\Core\Serialization\SerializationInterface.
+ */
+
+namespace Drupal\Core\Serialization;
+
+/**
+ * Defines an interface for serialization formats.
+ */
+interface SerializationInterface {
+
+  /**
+   * Encodes data into the serialization format.
+   *
+   * @param mixed $data
+   *   The data to encode.
+   *
+   * @return string
+   *   The encoded data.
+   */
+  public static function encode($data);
+
+  /**
+   * Decodes data from the serialization format.
+   *
+   * @param string $raw
+   *   The raw data string to decode.
+   *
+   * @return mixed
+   *   The decoded data.
+   */
+  public static function decode($raw);
+
+  /**
+   * Returns the file extension for this serialization format.
+   *
+   * @return string
+   *   The file extension, without leading dot.
+   */
+  public static function getFileExtension();
+
+}
diff --git a/core/modules/system/lib/Drupal/system/Tests/KeyValueStore/DatabaseStorageExpirableTest.php b/core/modules/system/lib/Drupal/system/Tests/KeyValueStore/DatabaseStorageExpirableTest.php
index 5f7365d..d53b914 100644
--- a/core/modules/system/lib/Drupal/system/Tests/KeyValueStore/DatabaseStorageExpirableTest.php
+++ b/core/modules/system/lib/Drupal/system/Tests/KeyValueStore/DatabaseStorageExpirableTest.php
@@ -35,7 +35,10 @@ protected function setUp() {
       ->addArgument('default');
     $this->container
       ->register('keyvalue.expirable.database', 'Drupal\Core\KeyValueStore\KeyValueDatabaseExpirableFactory')
+      ->addArgument(new Reference('serialization.phpserialize'))
       ->addArgument(new Reference('database'));
+    $this->container
+      ->register('serialization.phpserialize', 'Drupal\Core\Serialization\PhpSerialize');
     $this->settingsSet('keyvalue_expirable_default', 'keyvalue.expirable.database');
   }
 
diff --git a/core/modules/system/lib/Drupal/system/Tests/KeyValueStore/DatabaseStorageTest.php b/core/modules/system/lib/Drupal/system/Tests/KeyValueStore/DatabaseStorageTest.php
index ad286d6..3442530 100644
--- a/core/modules/system/lib/Drupal/system/Tests/KeyValueStore/DatabaseStorageTest.php
+++ b/core/modules/system/lib/Drupal/system/Tests/KeyValueStore/DatabaseStorageTest.php
@@ -34,7 +34,10 @@ protected function setUp() {
       ->addArgument('default');
     $this->container
       ->register('keyvalue.database', 'Drupal\Core\KeyValueStore\KeyValueDatabaseFactory')
+      ->addArgument(new Reference('serialization.phpserialize'))
       ->addArgument(new Reference('database'));
+    $this->container
+      ->register('serialization.phpserialize', 'Drupal\Core\Serialization\PhpSerialize');
     $this->settingsSet('keyvalue_default', 'keyvalue.database');
   }
 
diff --git a/core/modules/user/lib/Drupal/user/TempStoreFactory.php b/core/modules/user/lib/Drupal/user/TempStoreFactory.php
index 28d4147..7574ba7 100644
--- a/core/modules/user/lib/Drupal/user/TempStoreFactory.php
+++ b/core/modules/user/lib/Drupal/user/TempStoreFactory.php
@@ -10,6 +10,7 @@
 use Drupal\Core\Database\Connection;
 use Drupal\Core\KeyValueStore\DatabaseStorageExpirable;
 use Drupal\Core\Lock\LockBackendInterface;
+use Drupal\Core\Serialization\SerializationInterface;
 
 /**
  * Creates a key/value storage object for the current user or anonymous session.
@@ -17,6 +18,13 @@
 class TempStoreFactory {
 
   /**
+   * The serialization class to use.
+   *
+   * @var \Drupal\Core\Serialization\SerializationInterface
+   */
+  protected $serializer;
+
+  /**
    * The connection object used for this data.
    *
    * @var \Drupal\Core\Database\Connection $connection
@@ -33,12 +41,15 @@ class TempStoreFactory {
   /**
    * Constructs a Drupal\user\TempStoreFactory object.
    *
+   * @param \Drupal\Core\Serialization\SerializationInterface $serializer
+   *   The serialization class to use.
    * @param \Drupal\Core\Database\Connection $connection
    *   The connection object used for this data.
    * @param \Drupal\Core\Lock\LockBackendInterface $lockBackend
    *   The lock object used for this data.
    */
-  function __construct(Connection $connection, LockBackendInterface $lockBackend) {
+  function __construct(SerializationInterface $serializer, Connection $connection, LockBackendInterface $lockBackend) {
+    $this->serializer = $serializer;
     $this->connection = $connection;
     $this->lockBackend = $lockBackend;
   }
@@ -65,7 +76,7 @@ function get($collection, $owner = NULL) {
     }
 
     // Store the data for this collection in the database.
-    $storage = new DatabaseStorageExpirable($collection, $this->connection);
+    $storage = new DatabaseStorageExpirable($collection, $this->serializer, $this->connection);
     return new TempStore($storage, $this->lockBackend, $owner);
   }
 

diff --git a/core/lib/Drupal/Core/KeyValueStore/DatabaseStorageExpirable.php b/core/lib/Drupal/Core/KeyValueStore/DatabaseStorageExpirable.php
index fb0b8b0..f4923cf 100644
--- a/core/lib/Drupal/Core/KeyValueStore/DatabaseStorageExpirable.php
+++ b/core/lib/Drupal/Core/KeyValueStore/DatabaseStorageExpirable.php
@@ -10,6 +10,7 @@
 use Drupal\Core\DestructableInterface;
 use Drupal\Core\Database\Connection;
 use Drupal\Core\Database\Query\Merge;
+use Drupal\Core\Serialization\SerializationInterface;
 
 /**
  * Defines a default key/value store implementation for expiring items.
@@ -20,13 +21,6 @@
 class DatabaseStorageExpirable extends DatabaseStorage implements KeyValueStoreExpirableInterface, DestructableInterface {
 
   /**
-   * The connection object for this storage.
-   *
-   * @var \Drupal\Core\Database\Connection
-   */
-  protected $connection;
-
-  /**
    * Flag indicating whether garbage collection should be performed.
    *
    * When this flag is TRUE, garbage collection happens at the end of the
@@ -44,16 +38,15 @@ class DatabaseStorageExpirable extends DatabaseStorage implements KeyValueStoreE
    *
    * @param string $collection
    *   The name of the collection holding key and value pairs.
    * @param array $options
    *   An associative array of options for the key/value storage collection.
    *   Keys used:
    *   - connection: (optional) The database connection to use for storing the
    *     data. Defaults to the current connection.
    *   - table: (optional) The name of the SQL table to use. Defaults to
    *     key_value_expire.
    */
-  public function __construct($collection, Connection $connection, $table = 'key_value_expire') {
-    parent::__construct($collection, $connection, $table);
+  public function __construct($collection, SerializationInterface $serializer, Connection $connection, $table = 'key_value_expire') {
+    parent::__construct($collection, $serializer, $connection, $table);
   }
 
   /**
@@ -78,7 +71,7 @@ public function getMultiple(array $keys) {
         ':keys' => $keys,
         ':collection' => $this->collection,
       ))->fetchAllKeyed();
-    return array_map('unserialize', $values);
+    return array_map(array($this->serializer, 'decode'), $values);
   }
 
   /**
@@ -91,7 +84,7 @@ public function getAll() {
         ':collection' => $this->collection,
         ':now' => REQUEST_TIME
       ))->fetchAllKeyed();
-    return array_map('unserialize', $values);
+    return array_map(array($this->serializer, 'decode'), $values);
   }
 
   /**
@@ -107,7 +100,7 @@ function setWithExpire($key, $value, $expire) {
         'collection' => $this->collection,
       ))
       ->fields(array(
-        'value' => serialize($value),
+        'value' => $this->serializer->encode($value),
         'expire' => REQUEST_TIME + $expire,
       ))
       ->execute();
@@ -124,7 +117,7 @@ function setWithExpireIfNotExists($key, $value, $expire) {
       ->insertFields(array(
         'collection' => $this->collection,
         'name' => $key,
-        'value' => serialize($value),
+        'value' => $this->serializer->encode($value),
         'expire' => REQUEST_TIME + $expire,
       ))
       ->condition('collection', $this->collection)
@@ -44,16 +38,15 @@ class DatabaseStorageExpirable extends DatabaseStorage implements KeyValueStoreE
    * @param string $collection
    *   The name of the collection holding key and value pairs.
-   * @param array $options
-   *   An associative array of options for the key/value storage collection.
-   *   Keys used:
-   *   - connection: (optional) The database connection to use for storing the
-   *     data. Defaults to the current connection.
-   *   - table: (optional) The name of the SQL table to use. Defaults to
-   *     key_value_expire.
+   * @param \Drupal\Core\Serialization\SerializationInterface $serializer
+   *   The serialization class to use.
+   * @param \Drupal\Core\Database\Connection $connection
+   *   The database connection to use.
+   * @param string $table
+   *   The name of the SQL table to use, defaults to key_value_expire.
    */